%Initializing the heating system
%By Jonas Magnussen (2011)

% 4 transfer functions in the heat exchanger
%Defining relative heating and cooling
%            theta_tb - theta_fb                theta_dhi - theta_dho
%epsilon_h = ---------------------, epsilon_c = --------------------
%            theta_dhi - theta_fb                theta_dhi - theta_fb
%tb - to building
%fb - from building
%dhi - district heating in
%dho - district heating out

%Design temperatures
theta_tb = 78;
theta_dhi = 95;
theta_fb = 48;

%Fluid amounts [m^3/s]
water_flow_dh = 0.000657;

water_flow_b = 0.000657;

%Water out temperature
theta_dho = theta_dhi - (water_flow_b/water_flow_dh)*(theta_tb - theta_fb);

%Relative heating and cooling
e_h_dh = (theta_tb - theta_fb)/(theta_dhi - theta_fb);
e_c_dh = (theta_dhi - theta_dho)/(theta_dhi - theta_fb);

%Masses
m_met_dh = 150;
m_w_dh = 0.00657*density_water;
m_batt_dh = m_w_dh + m_met_dh;

spec_heat_cap_metal_dh = 900;

%Specific heat capacity battery

%Gains
K_bb = 1 - e_h_dh;
K_bdh = e_c_dh;
K_dhb = e_h_dh;
K_dhdh = 1 - e_c_dh;
T_batt_dh = (1 - e_c_dh)*(m_met_dh*spec_heat_cap_metal_dh)/(water_flow_dh*density_water*spec_heat_cap_water);

K_bb_s = T_batt_dh*((1 - e_c_dh - e_h_dh)/(1 - e_c_dh));

% Shunt connection
R_RP_dh = 50;
R_SP_dh = 50;
dp_rp_dh = 1736;
dp_sp_dh = 1736;
dp_r_dh = 300;
dp_s_dh = 150;

%Controller
K_p_dh = 0.01;

T_i_dh = 1*minute;
K_i_dh = 1/T_i_dh;

%Energy
calculate_energy = water_flow_dh*spec_heat_cap_water*density_water;