%Initialization values used during the linearization
%By Jonas Magnussen (2011)

day = 86400;
hour = 60*60;
minute = 60;

tstart = 0;
tstop = 1*day;
tstep = minute/6;

%% Room A
%Room spec
wall_depth_6A = 0.360;
wall_depth_CA = 0.122;
room_height_A = 3;
room_length_A = 4.5;
room_width_A = 2.5;
wall_area_6A = (2*room_width_A + room_length_A)*room_height_A; %3 walls towards outdoor
wall_area_CA = room_length_A*room_height_A;                    %1 wall towards room C
room_area_A = room_length_A*room_width_A;
room_volume_A = room_area_A*room_height_A;
window_area_A = 0.258*room_area_A;
radiator_area_A = 0.5*2;
radiator_depth_A = 0.01;
%Specific values
%Air
density_air = 1.2;
spec_heat_cap_air = 1005;
%Water
spec_heat_cap_water = 4200;
density_water = 1000;
%Metal (67 % Alu, 33 % Cu)
spec_heat_cap_metal = 0.67*900 + 0.33*390;
density_metal = 0.67*2700 + 0.33*8900;

%Outer wall material: brick (30,56%), gypsum (6,11%), air (8,33%) &
%insulation (55%)
density_brick = 1800;
spec_heat_cap_brick = 830;

density_gypsum = 900;
spec_heat_cap_gypsum = 800;
%Insulation: rock wool
density_insulation = 31;
spec_heat_cap_insulation = 1030;

density_6A = 0.3056*density_brick + 0.0611*density_gypsum + 0.0833*density_air + 0.55*density_insulation;
spec_heat_cap_6A = 0.3056*spec_heat_cap_brick + 0.0611*spec_heat_cap_gypsum + 0.0833*spec_heat_cap_air + 0.55*spec_heat_cap_insulation;

%District heating
density_steel_rad = 7750;
spec_heat_cap_rad = 460;
pipe_dimention_A = 0.02;
area_pipe_A = pi*(pipe_dimention_A/2)^2;
water_speed_A = 0.25;
w_incoming_water_A = area_pipe_A*water_speed_A;

%Wall towards room C material: gypsum (18,03%), air (40,98%) & insulation (40,98%)
density_CA = 0.1803*density_gypsum + 0.4098*density_air + 0.4098*density_insulation;
spec_heat_cap_CA = 0.1803*spec_heat_cap_gypsum + 0.4098*spec_heat_cap_air + 0.4098*spec_heat_cap_insulation;

%Convective: Outside air -> mass centre wall (Towards outdoor)
U_A6_outer = 0.25;
R_16A = 1/(wall_area_6A*U_A6_outer);

%Convective: Mass centre wall -> inner wall surface
U_A6_inner = 0.5;
R_26A =  1/(wall_area_6A*U_A6_inner);

%Convective: Inner wall surface (towards room C) -> mass centre wall (Towards room C)
U_AC_outer = 0.5;
R_2CA = 1/(wall_area_CA*U_A6_outer);

%Convective: Inner wall surface (towards outdoor) -> room air
h_36A = 3;
R_36A = 1/(h_36A*wall_area_6A);

%Convective: Inner wall surface (towards room C) -> room air
h_3CA = 3;

R_3CA = 1/(h_3CA*wall_area_CA);

%Radiation: Inner window surface -> inner wall surface (towards outdoor and room C)
h_4A = 5;
R_4A = 1/(h_4A*window_area_A);

%Convective: Inner window surface -> room air
h_wA = 3;
R_5A = 1/(h_wA*window_area_A);

%Convective: Outdoor air -> inner window surface
U_wA = 1.2;
R_6A = 1/(U_wA*window_area_A) - 1/(1/R_4A + 1/R_5A);

%Convective: Ventilation -> room air
ventilation_amount_per_hour_A = 8.6;
w_vent_A = (density_air*room_area_A)*ventilation_amount_per_hour_A/hour;
R_7A = 1/(w_vent_A*spec_heat_cap_air);

%Convective: Infiltration -> room air
air_exchange_per_hour_infil = 1.5;
w_i = (density_air*room_volume_A)*air_exchange_per_hour_infil/hour;
R_8A = 1/(w_i*spec_heat_cap_air);

%Radiation: Radiator -> inner wall surface (towards outdoor and room C)
h_rad_A = 7;
R_9A = 1/(h_rad_A*radiator_area_A);

%Radiation: Radiator -> room air (=Prod W)
Radiator_effect_A = 2000; %W
theta_delta = 30; %Difference between radiator temp. and room air temp.
R_10A = theta_delta/Radiator_effect_A;

%resultant resistors
R_1tA = 1/((1/R_26A) + (1/R_16A));
R_2tA = 1/((1/R_36A) + (1/R_3CA) + (1/R_5A) + (1/R_7A) + (1/R_8A) + (1/R_10A));% den vanlige
% R_2tA = 1/((1/R_36A) + (1/R_3CA) + (1/R_5A) + (1/R_8A) + (1/R_10A));% den vanlige

% R_2tA = 1/((1/R_36A) + (1/R_36A) + (1/R_5A) + (1/R_8A) + (1/R_10A)); %linmod 1 rom som funker 
R_3tA = 1/((1/R_2CA) + (1/R_2CA));
R_4tA = 1/((1/R_26A) + (1/R_36A) + (1/R_4A) + (1/R_9A));
R_5tA = 1/((1/R_2CA) + (1/R_3CA) + (1/R_4A) + (1/R_9A));
R_6tA = 1/((1/R_4A) + (1/R_4A) + (1/R_5A) + (1/R_6A)); %den vanlige
% R_6tA = 1/((1/R_4A) + (1/R_4A) + (1/R_5A) + (1/R_6A)); %linmod 1 rom som funker 


%Noise
%Radiation from people: 4,8 [W/m^2]
W_people_A = 4.8*room_area_A;
radius_people_A = 0.406;
height_people_A = 1.86;
Area_people_A = 2*pi*radius_people_A*(height_people_A + radius_people_A);
number_of_people_A = 1;

h_people_A = 3;
k_people_A = 0.2;

R_people_A = 1/(number_of_people_A*Area_people_A*h_people_A);

%Radiation from lighting: 11,6 [W/m^2]
W_lighting_A = 11.6*room_area_A;
radius_lights_A = 0.025;
height_lights_A = 1.5;
Area_lights_A = 2*pi*radius_lights_A*(height_lights_A + radius_lights_A);
number_of_lights_A = 3;

h_lights_A = 6;
k_lights_A = 0.3;

R_lights_A = 1/(number_of_lights_A*Area_lights_A*h_lights_A);
%Radiation from equipment: 14 [W/m^2]
W_equip_A = 14*room_area_A;

Area_equip_A = 0.5;
number_of_equip_A = 1;

h_equip_A = 5;
k_equip_A = 0.4;

R_equip_A = 1/(number_of_equip_A*Area_equip_A*h_equip_A);

noise_sampletime_A = 60;
noise_uncertainty_A = 80;
%Wall heat storage capacity: Walls towards outdoor air
%Heat capacity, C = volume * density * specific heat capacity
C_tot_6A = wall_depth_6A*wall_area_6A*density_6A*spec_heat_cap_6A;

%Wall heat storage capacity: Walls towards outdoor air
C_tot_CA = wall_depth_CA*wall_area_CA*density_CA*spec_heat_cap_CA;

%Air heat storage
C_main_air_A = room_height_A*room_area_A*density_air*spec_heat_cap_air;
C_air_inventory_A = C_main_air_A*20;
C_air_A = C_air_inventory_A + C_main_air_A;

%Radiator heat storage
C_rad_A = radiator_area_A*radiator_depth_A*spec_heat_cap_rad*density_steel_rad;
% C_rad_A = radiator_area_A*radiator_depth_A*10000;


%Initial temperatures
T_init_air_A = 17;
T_init_wall_6_A = 15;
T_init_wall_C_A = 15;
theta_2A_desired = 21;
theta_init_window_A = 15;
theta_vent_init_A = 15;

%Time constants
T_rad_A = 200;

T_1A = C_tot_6A*R_1tA;
T_2A = C_air_A*R_2tA;
T_3A = C_tot_CA*R_3tA;

%Radiator
%Indoor temperature controller
T_I_A = minute*18;
K_p_A = 1;
K_I_A = 1/T_I_A;
T_d_A = 500;

T_heat_supply = 8*minute;

%% Room B
%Room spec
wall_depth_6B = 0.360;
wall_depth_CB = 0.122;
room_height_B = 3;
room_length_B = 2.5;
room_width_B = 6;
wall_area_6B = (2*room_length_B + room_width_B)*room_height_B; %3 walls towards outdoor
wall_area_CB = room_width_B*room_height_B;                    %1 wall towards room C
room_area_B = room_length_B*room_width_B;
room_volume_B = room_area_B*room_height_B;
window_area_B = 0.258*room_area_B;
radiator_area_B = 0.5*2;
radiator_depth_B = 0.01;

pipe_dimention_B = 0.02;
area_pipe_B = pi*(pipe_dimention_B/2)^2;
water_speed_B = 0.5;
w_incoming_water_B = area_pipe_B*water_speed_B;

%Outer wall material: brick (30,56%), gypsum (6,11%), air (8,33%) &
density_6B = 0.3056*density_brick + 0.0611*density_gypsum + 0.0833*density_air + 0.55*density_insulation;
spec_heat_cap_6B = 0.3056*spec_heat_cap_brick + 0.0611*spec_heat_cap_gypsum + 0.0833*spec_heat_cap_air + 0.55*spec_heat_cap_insulation;

%Wall towards room C material: gypsum (18,03%), air (40,98%) & insulation (40,98%)
density_CB = 0.1803*density_gypsum + 0.4098*density_air + 0.4098*density_insulation;
spec_heat_cap_CB = 0.1803*spec_heat_cap_gypsum + 0.4098*spec_heat_cap_air + 0.4098*spec_heat_cap_insulation;

%Convective: Outside air -> mass centre wall (Towards outdoor)
U_6B_outer = 0.25;
R_16B = 1/(wall_area_6B*U_6B_outer);

%Convective: Mass centre wall -> inner wall surface
U_6B_inner = 0.5;
R_26B =  1/(wall_area_6B*U_6B_inner);

%Convective: Inner wall surface (towards room C) -> mass centre wall (Towards room C)
U_CB_outer = 0.5;
R_2CB = 1/(wall_area_CB*U_CB_outer);

%Convective: Inner wall surface (towards outdoor) -> room air
h_36B = 3;
R_36B = 1/(h_36B*wall_area_6B);

%Convective: Inner wall surface (towards room C) -> room air
h_3CB = 3;
R_3CB = 1/(h_3CB*wall_area_CB);

%Radiation: Inner window surface -> inner wall surface (towards outdoor and room C)
h_4B = 5;
R_4B = 1/(h_4B*window_area_B);

%Convective: Inner window surface -> room air
h_wB = 3;
R_5B = 1/(h_wB*window_area_B);

%Convective: Outdoor air -> inner window surface
U_wB = 1.2;
R_6B = 1/(U_wB*window_area_B) - 1/(1/R_4B + 1/R_5B);

%Convective: Ventilation -> room air
ventilation_amount_per_hour_B = 8.6;
w_vent_B = (density_air*room_area_B)*ventilation_amount_per_hour_B/hour;
R_7B = 1/(w_vent_B*spec_heat_cap_air);

%Convective: Infiltration -> room air
air_exchange_per_hour_infil_B = 1.5;
w_i_B = (density_air*room_volume_B)*air_exchange_per_hour_infil_B/hour;
R_8B = 1/(w_i_B*spec_heat_cap_air);

%Radiation: Radiator -> inner wall surface (towards outdoor and room C)
h_rad_B = 7;
R_9B = 1/(h_rad_B*radiator_area_B);

%Radiation: Radiator -> room air (=Prod W)
Radiator_effect_B = 2100; %W
theta_delta = 30; %Difference between radiator temp. and room air temp.
R_10B = theta_delta/Radiator_effect_B;

%resultant resistors
R_1tB = 1/((1/R_26B) + (1/R_16B));
R_2tB = 1/((1/R_36B) + (1/R_3CB) + (1/R_5B) + (1/R_7B) + (1/R_8B) + (1/R_10B)); %vanlig
% R_2tB = 1/((1/R_36B) + (1/R_3CB) + (1/R_5B) + (1/R_8B) + (1/R_10B));

R_3tB = 1/((1/R_2CB) + (1/R_2CB));
R_4tB = 1/((1/R_26B) + (1/R_36B) + (1/R_4B) + (1/R_9B));
R_5tB = 1/((1/R_2CB) + (1/R_3CB) + (1/R_4B) + (1/R_9B));
R_6tB = 1/((1/R_4B) + (1/R_4B) + (1/R_5B) + (1/R_6B));

%Noise
%Radiation from people: 4,8 [W/m^2]
W_people_B = 4.8*room_area_B;
h_people_B = 3;
k_people_B = 0.6;

%Radiation from lighting: 11,6 [W/m^2]
W_lighting_B = 11.6*room_area_B;
h_lights_B = 6;
k_lights_B = 0.7;

%Radiation from equipment: 14 [W/m^2]
W_equip_B = 14*room_area_B;
h_equip_B = 5;
k_equip_B = 0.7;

noise_sampletime_B = 60;
noise_uncertainty_B = 150;
%Wall heat storage capacity: Walls towards outdoor air
%Heat capacity, C = volume * density * specific heat capacity
C_tot_6B = wall_depth_6B*wall_area_6B*density_6B*spec_heat_cap_6B;

%Wall heat storage capacity: Walls towards outdoor air
C_tot_CB = wall_depth_CB*wall_area_CB*density_CB*spec_heat_cap_CB;

%Air heat storage
C_main_air_B = room_height_B*room_area_B*density_air*spec_heat_cap_air;
C_air_inventory_B = C_main_air_B*30;
C_air_B = C_air_inventory_B + C_main_air_B;

%Radiator
C_rad_B = radiator_area_B*radiator_depth_B*spec_heat_cap_rad*density_steel_rad;


%Initial temperatures
T_init_air_B = 17;
T_init_wall_6_B = 15;
T_init_wall_C_B = 15;
theta_2B_desired = 21;
theta_init_window_B = 15;
theta_vent_init_B = 15;


%Time constants
T_rad_B = 200;

T_1B = C_tot_6B*R_1tB;
T_2B = C_air_B*R_2tB;
T_3B = C_tot_CB*R_3tB;

%Indoor temperature controller
tau_process = 3*minute;
K_p_B = 1;
T_i_B = 1/tau_process;
T_d_B = 0;

%% Room C
%Room spec
wall_depth_6C = 0.360;
wall_depth_ABC = 0.122;
room_height_C = 3;
room_length_C = 4.5;
room_width_C = 6;
wall_area_6C = (room_length_C + room_width_C)*room_height_C; %vanlig
% wall_area_6C = (2*room_length_C + room_width_C)*room_height_C; %3 walls towards outdoor
wall_area_BC = room_width_C*room_height_C;                    %1 wall towards room C
wall_area_AC = room_length_C*room_height_C; 
room_area_C = room_length_C*room_width_C;
room_volume_C = room_area_C*room_height_C;
window_area_C = 0.258*room_area_C;
radiator_area_C = 0.75*2;
radiator_depth_C = 0.01;

pipe_dimention_C = 0.02;
area_pipe_C = pi*(pipe_dimention_C/2)^2;
water_speed_C = 0.5;
w_incoming_water_C = area_pipe_C*water_speed_C;

%Outer wall material: brick (30,56%), gypsum (6,11%), air (8,33%) &
%insulation (55%)

density_6C = 0.3056*density_brick + 0.0611*density_gypsum + 0.0833*density_air + 0.55*density_insulation;
spec_heat_cap_6C = 0.3056*spec_heat_cap_brick + 0.0611*spec_heat_cap_gypsum + 0.0833*spec_heat_cap_air + 0.55*spec_heat_cap_insulation;

%Wall towards room C material: gypsum (18,03%), air (40,98%) & insulation (40,98%)
density_ABC = 0.1803*density_gypsum + 0.4098*density_air + 0.4098*density_insulation;
spec_heat_cap_ABC = 0.1803*spec_heat_cap_gypsum + 0.4098*spec_heat_cap_air + 0.4098*spec_heat_cap_insulation;

%Convective: Outside air -> mass centre wall (Towards outdoor)
U_6C_outer = 0.25;
R_16C = 1/(wall_area_6C*U_6C_outer);

%Convective: Mass centre wall -> inner wall surface
U_6C_inner = 0.5;
R_26C =  1/(wall_area_6C*U_6C_inner);

%Convective: Inner wall surface (towards room A) -> mass centre wall (Towards room A)
R_2AC = R_2CA;

%Convective: Inner wall surface (towards room B) -> mass centre wall (Towards room B)
R_2BC = R_2CB;

%Convective: Inner wall surface (towards outdoor) -> room air
h_36C = 3;
R_36C = 1/(h_36C*wall_area_6C);

%Convective: Inner wall surface (towards room A) -> room air
h_3AC = 3;
R_3AC = 1/(h_3AC*wall_area_AC);

%Convective: Inner wall surface (towards room B) -> room air
h_3BC = 3;
R_3BC = 1/(h_3BC*wall_area_BC);

%Radiation: Inner window surface -> inner wall surface (towards outdoor and room C)
h_4C = 5;
R_4C = 1/(h_4C*window_area_C);

%Convective: Inner window surface -> room air
h_wC = 3;
R_5C = 1/(h_wC*window_area_C);

%Convective: Outdoor air -> inner window surface
U_wC = 1.2;
R_6C = 1/(U_wC*window_area_C) - 1/(1/R_4C + 1/R_5C);

%Convective: Ventilation -> room air
ventilation_amount_per_hour_C = 8.6;
w_vent_C = (density_air*room_area_C)*ventilation_amount_per_hour_C/hour;
R_7C = 1/(w_vent_C*spec_heat_cap_air);

%Convective: Infiltration -> room air
air_exchange_per_hour_infil_C = 1.5;
w_i_C = (density_air*room_volume_C)*air_exchange_per_hour_infil_C/hour;
R_8C = 1/(w_i_C*spec_heat_cap_air);

%Radiation: Radiator -> inner wall surface (towards outdoor and room C)
h_rad_C = 7;
R_9C = 1/(h_rad_C*radiator_area_C);

%Radiation: Radiator -> room air (=Prod W)
Radiator_effect_C = 3200; %W
theta_delta = 30; %Difference between radiator temp. and room air temp.
R_10C = theta_delta/Radiator_effect_C;

%resultant resistors
R_1tC = 1/((1/R_26C) + (1/R_16C));                              %Wall mass out
R_2tC = 1/((1/R_36C) + (1/R_3AC) + (1/R_3BC) + (1/R_5C)...
        + (1/R_7C) + (1/R_8C) + (1/R_10C));                     %Room air vanlig
% R_2tC = 1/((1/R_36C) + (1/R_3AC) + (1/R_3BC) + (1/R_5C)...
%         + (1/R_8C) + (1/R_10C));                     %Room air

R_3tAC = R_3tA;                                                 %Wall mass tow. A
R_3tBC = R_3tB;                                                 %Wall mass tow. B
R_4tC = 1/((1/R_26C) + (1/R_36C) + (1/R_4C) + (1/R_9C));        %Wall surface out
R_5tAC = 1/((1/R_2AC) + (1/R_3AC) + (1/R_4C) + (1/R_9C));       %Wall surface A
R_5tBC = 1/((1/R_2BC) + (1/R_3BC) + (1/R_4C) + (1/R_9C));       %Wall surface B
R_6tC = 1/((1/R_4C) + (1/R_4C) + (1/R_4C) + (1/R_5C) + (1/R_6C));%Window surface

%Noise
%Radiation from people: 4,8 [W/m^2]
W_people_C = 4.8*room_area_C;
h_people_C = 3;
k_people_C = 0.8;

%Radiation from lighting: 11,6 [W/m^2]
W_lighting_C = 11.6*room_area_C;
h_lights_C = 6;
k_lights_C = 0.7;

%Radiation from equipment: 14 [W/m^2]
W_equip_C = 14*room_area_C;
h_equip_C = 5;
k_equip_C = 0.7;

noise_sampletime_C = 60;
noise_uncertainty_C = 300;
%Wall heat storage capacity: Walls towards outdoor air
%Heat capacity, C = volume * density * specific heat capacity
C_tot_6C = wall_depth_6C*wall_area_6C*density_6C*spec_heat_cap_6C;

%Wall heat storage capacity: Walls towards room A
C_tot_AC = wall_depth_ABC*wall_area_AC*density_ABC*spec_heat_cap_ABC;

%Wall heat storage capacity: Walls towards room B
C_tot_BC = wall_depth_ABC*wall_area_BC*density_ABC*spec_heat_cap_ABC;

%Air heat storage
C_main_air_C = room_height_C*room_area_C*density_air*spec_heat_cap_air;
C_air_inventory_C = C_main_air_C*50;
C_air_C = C_air_inventory_C + C_main_air_C;

%Radiator
C_rad_C = radiator_area_C*radiator_depth_C*spec_heat_cap_rad*density_steel_rad;


%Initial temperatures
T_init_air_C = 17;
T_init_wall_6_C = 15;
T_init_wall_A_C = 15;
T_init_wall_B_C = 15;
theta_2C_desired = 21;
theta_init_window_C = 15;
rad_temp_init = 30;
theta_vent_init_C = 20;

%Time constants
T_rad_C = 200;

T_1C = C_tot_6C*R_1tC;
T_2C = C_air_C*R_2tC;
T_3C = C_tot_AC*R_3tAC;
T_4C = C_tot_BC*R_3tBC;

%Indoor temperature controller
tau_process = 5.5*minute;
K_p_C = 1;
T_i_C = 1/tau_process;
T_d_C = 0;

T_heat_supply = 8*minute;

%% Ventilation
% 4 transfer functions in the heat exchanger
%Defining relative heating and cooling
%            theta_ao - theta_ai                theta_wi - theta_wo
%epsilon_h = ---------------------, epsilon_c = --------------------
%            theta_wi - theta_ai                theta_wi - theta_ai

%Design temperatures
% theta_ao = 35.8;
% theta_wi = 65;
% theta_ai = 18;
theta_ao = 45;
theta_wi = 70;
theta_ai = 18;

%Fluid amounts
air_flow = 0.857;    %[m^3/s]
% air_flow = 0.457;
% water_flow = 0.0005; %[m^3/s]
water_flow = 0.000205;
% water_flow = 0.0009;

%Water out temperature
theta_wo = theta_wi - ((air_flow*density_air*spec_heat_cap_air)...
                        /(water_flow*density_water*spec_heat_cap_water))...
                        *(theta_ao - theta_ai);

%Relative heating and cooling
e_h = (theta_ao - theta_ai)/(theta_wi - theta_ai);
e_c = (theta_wi - theta_wo)/(theta_wi - theta_ai);

%Masses
m_met = 28;
m_w = 0.002*density_water;
% m_w = 0.009*density_water;
m_batt = m_w + m_met;

%Specific heat capacity battery
% spec_heat_cap_batt = (spec_heat_cap_water*m_w + spec_heat_cap_metal*m_met)/m_batt;

%Gains
K_aa = 1 - e_h;
K_aw = e_c;
K_wa = e_h;
K_ww = 1 - e_c;
% T_batt = (1 - e_c)*(m_w*spec_heat_cap_water + m_met*spec_heat_cap_metal)/(water_flow*density_water*spec_heat_cap_water);
T_batt = (1 - e_c)*(m_met*spec_heat_cap_metal)/(water_flow*density_water*spec_heat_cap_water);

K_aa_s = T_batt*((1 - e_c - e_h)/(1 - e_c));

% Shunt connection
R_RP = 50;
R_SP = 50;
dp_rp = 1736;
dp_sp = 1736;
dp_r = 300;
dp_s = 150;

%Controller
K_p_V = 0.01;

T_i_V = 1*minute;
K_i_V = 1/T_i_V;


%% District heating
%%Water temperature control

T_delay = 8*minute;
T_desired_response = 4*minute;

K_p_water = T_delay/T_desired_response;
K_i_water = 1/T_desired_response;

%Indoor temperature controller
tau_process = 0.6*minute;
K_p = 70;
T_i = 1/tau_process;
T_d = 0;

%% Energy (fra prosjekt. Dette m endres)
%Radiators
%P = U^2/R -> U = delta_theta = theta_4 - theta_2, R = R_9 eller R_10

% %Constants needed in order to calculate energy efficiency
% c_w = 4200;
% rho_w = 1000;
% pipe_dimention = 0.08;
% area_pipe = pi*(pipe_dimention/2)^2;
% water_speed = 1;
% w_incoming_water = area_pipe*water_speed;
% calculate_energy = w_incoming_water*c_w*rho_w;
