%Define an MPC problem
%By Jonas Magnussen (2011)

[numberOfRows_A_d, numberOfColumns_A_d] = size(A_d);
[numberOfRows_B_d_u, numberOfColumns_B_d_u] = size(B_d_u);
[numberOfRows_B_d_d, numberOfColumns_B_d_d] = size(B_d_d);
[numberOfRows_C_d, numberOfColumns_C_d] = size(C_d);

A_row = zeros(H_p*numberOfRows_A_d, numberOfColumns_A_d);
C_z = zeros(H_p*numberOfRows_C_d, H_p*numberOfColumns_C_d);

A_B_row = zeros(H_p*numberOfRows_B_d_u, size(A_d*B_d_u, 2));
[numberOfRows_A_B_row, numberOfColumns_A_B_row] = size(A_B_row);

A_B_row_dis = zeros(H_p*numberOfRows_B_d_d, size(C_d*A_d*B_d_d, 2));
[numberOfRows_A_B_row_dis, numberOfColumns_A_B_row_dis] = size(A_B_row_dis);

A_B_diag = zeros(H_p*numberOfRows_B_d_u, size(A_d*B_d_u, 2)*H_u);
A_B_diag_dis = zeros(H_p*numberOfRows_B_d_d, size(C_d*A_d*B_d_d, 2)*H_p);

%this for-loop generates the C_z, Psi, Upsilon and Theta matrices
for i = 0:H_p-1
    C_z(numberOfRows_C_d*i + 1:numberOfRows_C_d*i + numberOfRows_C_d, numberOfColumns_C_d*i + 1:numberOfColumns_C_d*i + numberOfColumns_C_d) = C_d;
    A_row(i*numberOfRows_A_d + 1:i*numberOfRows_A_d + numberOfRows_A_d, 1:numberOfColumns_A_d) = A_d^(i+1);
    A_B_row_dis(i*numberOfRows_C_d + 1:i*numberOfRows_C_d + numberOfRows_C_d, 1:size(C_d*A_d*B_d_d, 2)) = C_d*(A_d^(i+1))*B_d_d;

    if i == 0
        A_B_row(1 + numberOfRows_B_d_u*i:numberOfRows_B_d_u + numberOfRows_B_d_u*i, 1:size(A_d*B_d_u, 2)) = A_d^(i)*B_d_u;
    else
        A_B_row(1 + numberOfRows_B_d_u*i:numberOfRows_B_d_u + numberOfRows_B_d_u*i, 1:size(A_d*B_d_u, 2)) = A_B_row(1 + numberOfRows_B_d_u*(i-1):numberOfRows_B_d_u + numberOfRows_B_d_u*(i-1), 1:size(A_d*B_d_u, 2))+ A_d^(i)*B_d_u;
    end
end

for j = 0:H_u-1
    A_B_diag(1 + numberOfRows_B_d_u*j:numberOfRows_A_B_row, 1 + numberOfColumns_B_d_u*j:numberOfColumns_B_d_u + numberOfColumns_B_d_u*j) = A_B_row(1:numberOfRows_A_B_row - j*numberOfRows_B_d_u, 1:numberOfColumns_B_d_u);
end

for t = 0:H_p-1
    A_B_diag_dis(1 + numberOfRows_B_d_d*t:numberOfRows_A_B_row_dis, 1 + numberOfColumns_B_d_d*t:numberOfColumns_B_d_d + numberOfColumns_B_d_d*t) = A_B_row_dis(1:numberOfRows_A_B_row_dis - t*numberOfRows_B_d_d, 1:numberOfColumns_B_d_d);
end

Psi = C_z*A_row;
Upsilon = C_z*A_B_row;
Theta = C_z*A_B_diag;

%Feedforward
Sigma = C_z*A_B_diag_dis;

%Generate constraint matrices

%input penalization
alternate_rowvec = zeros(H_u, 1);
F = zeros(2*H_u, H_u);
f = zeros(2*H_u, 1);
F_1 = ones(2*H_u, 1);

for n = 1:2*H_u
    alternate_rowvec(n, 1) = -1;
    f(n, 1) = -35;
    if (n > 1)
        if(alternate_rowvec(n - 1, 1) == -1)
            alternate_rowvec(n, 1) = 1;
            f(n, 1) = 80;
        else
            alternate_rowvec(n, 1) = -1;
            f(n, 1) = -35;
        end
    end
    F_1(n) = alternate_rowvec(n)*(-1);
end
for o = 0:H_u-1
    F(1 + 2*o:2*H_u, 1+o) = alternate_rowvec(1:2*H_u - o*2,1); 
end

%output penalization
Gamma = -1*eye(H_p*numberOfRows_C_d, H_p*numberOfRows_C_d);
[numberOfRows_Gamma, numberOfColumns_Gamma] = size(Gamma);
identity_gamma = [-1*eye(numberOfRows_C_d);
                     eye(numberOfRows_C_d)];

[numberOfRows_identity, numberOfColumns_identity] = size(identity_gamma);

W = zeros(2*H_u, H_u);                
w = zeros(2*H_u, 1);

alternate_W = [1;-1];
alternate_w = [5;5];
[numberOfRows_alternate, numberOfColumns_alternate] = size(alternate_W);

for r = 0:H_u-1
    W(1 + numberOfRows_alternate*r:numberOfRows_alternate + r*numberOfRows_alternate, 1 + numberOfColumns_alternate*r:numberOfColumns_alternate + numberOfColumns_alternate*r) = alternate_W;
    w(1 + numberOfRows_alternate*r:numberOfRows_alternate + r*numberOfRows_alternate, 1) = alternate_w;
end

%State constraints (night and day temperature)
g_1 = zeros(H_p*numberOfRows_C_d, 1);
g_2 = zeros(H_p*numberOfRows_C_d, 1);
for m = 0:H_p-1       
    g_1(m*numberOfRows_C_d+1:m*numberOfRows_C_d + numberOfRows_C_d, 1) = eps_day;
    g_2(m*numberOfRows_C_d+1:m*numberOfRows_C_d + numberOfRows_C_d, 1) = eps_night;
end

%Generate u-matrices
Heta = zeros(H_u, H_u);
chi = ones(H_u, 1);

for p = 0:H_u-1
    Heta(1 + p:H_u, 1+p) = chi(1:H_u - p,1);
end

%Generate penalization matrices

%Output penalization
Q_time_step = eye(size(C_d, 1))*0;
[numberOfRows_Q_time_step, numberOfColumns_Q_time_step] = size(Q_time_step);

Q = zeros(numberOfRows_Q_time_step*(H_p-H_w), numberOfColumns_Q_time_step*(H_p-H_w));
for p = 0:(H_p-1)
   Q(numberOfRows_Q_time_step*p + 1:numberOfRows_Q_time_step*p + numberOfRows_Q_time_step, numberOfColumns_Q_time_step*p + 1:numberOfColumns_Q_time_step*p + numberOfColumns_Q_time_step) = Q_time_step;
end

%Input change penalization
R = eye(H_u)*1;
  
%Input penalization
S = eye(H_u)*100;

%Generate matrices in optimazation problem
H = Theta'*Q*Theta + R + Heta'*S*Heta;
G_without_epsilon = 2*Theta'*Q;

Z = 2*Heta'*S*chi;