%Solving a quadratic optimization problem with slack variables
%By Jonas Magnussen(2011)
function optimizer_output = optimize_slack(vectors)

persistent optimizer_output_prev;
persistent optimizer_counter;
persistent quadprog_runned_status;


minute = 60;
hour = minute*minute;
seconds = vectors(length(vectors));
Cd_size = 4;
number_of_states = 17;

Ts = 5*minute;

H_p = 50;
H_u = 35;

sampleNUMBER = mod(floor(seconds/Ts), H_p*Ts);

x = mod(seconds, Ts);

choose_quadprog = mod(floor(seconds/hour), 24);
  
if seconds == 0 && isempty(optimizer_counter) == 1
    optimizer_counter = 0;
    quadprog_runned_status = 1;
end

if x == 0 && quadprog_runned_status ~= sampleNUMBER
    H_vec = vectors(1:(H_u*H_u), 1); 
    G_vec = vectors((((H_u*H_u)+1)):(((H_u*H_u))+H_u), 1);
    F_vec = vectors((((H_u*H_u)+H_u)+1):(((H_u*H_u))+H_u + (2*H_u*H_u)), 1);
    F_1_vec = vectors(((((H_u*H_u)+H_u)+ 2*H_u*H_u)+1):((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u)), 1);
    f_vec = vectors((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+1):(((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u)), 1);
    Gamma_vec = vectors((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u)) + 1:(((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size)), 1);
    g_vec_1 = vectors((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size))+1: (((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size) + H_p*Cd_size), 1);
    g_vec_2 = vectors((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size) + H_p*Cd_size)+1: (((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size) + (H_p*Cd_size)) + (H_p*Cd_size), 1);
    Theta_vec = vectors((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size) + 2*H_p*Cd_size)+1: ((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size) + 2*H_p*Cd_size) + Cd_size*H_p*H_u), 1);
    Psi_vec = vectors(((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size) + 2*H_p*Cd_size) + Cd_size*H_p*H_u) + 1:((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size) + 2*H_p*Cd_size) + Cd_size*H_p*H_u + Cd_size*H_p*number_of_states), 1);
    Upsilon_vec = vectors(((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size) + 2*H_p*Cd_size) + Cd_size*H_p*H_u + Cd_size*H_p*number_of_states) + 1:(((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size) + 2*H_p*Cd_size) + Cd_size*H_p*H_u + Cd_size*H_p*number_of_states) + Cd_size*H_p), 1);
    x_hat_vec = vectors((((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size) + 2*H_p*Cd_size) + Cd_size*H_p*H_u + Cd_size*H_p*number_of_states) + Cd_size*H_p) + 1:((((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size) + 2*H_p*Cd_size) + Cd_size*H_p*H_u + Cd_size*H_p*number_of_states) + Cd_size*H_p) + number_of_states), 1);
    C_d_vec = vectors((((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size) + 2*H_p*Cd_size) + Cd_size*H_p*H_u + Cd_size*H_p*number_of_states) + Cd_size*H_p + number_of_states) + 1:((((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size) + 2*H_p*Cd_size) + Cd_size*H_p*H_u + Cd_size*H_p*number_of_states) + Cd_size*H_p) + number_of_states + Cd_size*number_of_states), 1);
    F_F_vec = vectors((((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size) + 2*H_p*Cd_size) + Cd_size*H_p*H_u + Cd_size*H_p*number_of_states) + Cd_size*H_p + number_of_states + Cd_size*number_of_states) + 1:((((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size) + 2*H_p*Cd_size) + Cd_size*H_p*H_u + Cd_size*H_p*number_of_states) + Cd_size*H_p) + number_of_states + Cd_size*number_of_states + Cd_size*H_p), 1);
    W_vec = vectors((((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size) + 2*H_p*Cd_size) + Cd_size*H_p*H_u + Cd_size*H_p*number_of_states) + Cd_size*H_p + number_of_states + Cd_size*number_of_states + Cd_size*H_p) + 1:((((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size) + 2*H_p*Cd_size) + Cd_size*H_p*H_u + Cd_size*H_p*number_of_states) + Cd_size*H_p) + number_of_states + Cd_size*number_of_states + Cd_size*H_p + 2*H_u*H_u), 1);
    v_vec = vectors((((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size) + 2*H_p*Cd_size) + Cd_size*H_p*H_u + Cd_size*H_p*number_of_states) + Cd_size*H_p + number_of_states + Cd_size*number_of_states + Cd_size*H_p + 2*H_u*H_u) + 1:((((((((H_u*H_u))+(H_u) + (2*H_u*H_u)) + (2*H_u))+(2*H_u) + (H_p*Cd_size*H_p*Cd_size) + 2*H_p*Cd_size) + Cd_size*H_p*H_u + Cd_size*H_p*number_of_states) + Cd_size*H_p) + number_of_states + Cd_size*number_of_states + Cd_size*H_p + 2*H_u*H_u + 2*H_u), 1);
    u_previous = vectors(length(vectors)-1);

    H = vector2matrix(H_vec, H_u, H_u)*2;
    G = G_vec;
    F = vector2matrix(F_vec, 2*H_u, H_u);
    W = vector2matrix(W_vec, 2*H_u, H_u);
    
    %Feedforward
    F_F = vector2matrix(F_F_vec, H_p*Cd_size, 1);
    
    Gamma = vector2matrix(Gamma_vec, H_p*Cd_size, H_p*Cd_size);
    Theta = vector2matrix(Theta_vec, H_p*Cd_size, H_u);
    Psi = vector2matrix(Psi_vec, Cd_size*H_p, number_of_states);
    
    f_quadprog = F_1_vec*u_previous + f_vec;
    
    if choose_quadprog < 12
        g = g_vec_1;
    elseif choose_quadprog >= 12
        g = g_vec_2;
    end
    
    gamma_quadprog = -1*Gamma*(Psi*x_hat_vec + Upsilon_vec*u_previous + F_F) - g;

    omega = [f_quadprog; gamma_quadprog; v_vec];
    
    F_1 = F;
    F_2 = Gamma*Theta;
    
    Omega = [F_1;F_2; W];

    %Penalization of constraint violations
    output_weights = zeros(H_p*Cd_size, 1); 
    weights = [10^5, 10^5, 10^5, 10^5]';
    for i = 0:H_p-1
        output_weights(1 + i*Cd_size:Cd_size+i*Cd_size,1) = weights;
    end
    input_weigths = ones(H_u*2, 1)*10^inf;         
    delta_in_weight = ones(H_u*2, 1)*10^inf;

    hard_and_soft_weight = [input_weigths;output_weights;delta_in_weight];

    [x,~,~,~]=qpsoft(H,G,Omega,omega,[],[],[],hard_and_soft_weight,'1-norm');

    optimizer_output = x(1);
    optimizer_output_prev = optimizer_output;

    quadprog_runned_status = sampleNUMBER;
    optimizer_counter = 1;
else
    quadprog_runned_status = sampleNUMBER;
    optimizer_counter = 0;
    optimizer_output = optimizer_output_prev;   
end